==> mutual_credit-5.0.0-beta3.tar.gz: mutual_credit/src/Plugin/views/field/RunningBalance.php <==
<?php

namespace Drupal\mcapi\Plugin\views\field;

use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\mcapi\Element\WorthsView;
use Drupal\views\ResultRow;
use Drupal\views\ViewExecutable;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\Core\Entity\EntityTypeManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Field handler to provide running balance for a given transaction.
 *
 * @note reads from the transaction index table
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("transaction_running_balance")
 *
 * @todo make an option for this field to sort on, serial or xid, or created
 */
class RunningBalance extends Worth {

  private $txStorage;
  private $fAlias;

  /**
   * Constructs a Handler object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param EntityTypeManager $entity_type_manager
   *   The EntityTypeManager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->txStorage = $entity_type_manager->getStorage('mcapi_transaction');
  }

  /**
   * {@inheritdoc}
   */
  public function init(ViewExecutable $view, DisplayPluginBase $display, array &$options = NULL) {
    parent::init($view, $display, $options);
    $this->additional_fields['running_created'] = ['field' => 'created', 'table' =>  $this->table];
  }


  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('entity_type.manager')
    );
  }


  /**
   * {@inheritdoc}
   */
  public function query() {
    // Determine the wallet id for which we want the balance.
    // It could from one of two args, or a filter.
    $this->addAdditionalFields();
    $this->fAlias = $this->getFirstWalletFieldAlias();
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $transaction = $this->getEntity($values);
    if (!$transaction) {
      //something wrong here
      throw new \Exception('No entity in views result');
    }

    $vals = $this->getBalance($transaction, $values->{$this->fAlias}, $values->{$this->aliases['running_created']});
    $transaction->worth->setValue($vals);
    $options = [
      'label' => 'hidden',
      'context' => WorthsView::MODE_TRANSACTION,
      'settings' => [],
    ];
    return $transaction->worth->view($options);
  }

  /**
   * Add up all the balances until the current transactions.
   *
   * @param TransactionInterface $transaction
   *   The current transaction
   * @param string $fAlias
   *   The alias of the field name to use
   * @param int $before
   *   The
   */
  private function getBalance($transaction, $fAlias, $before) {
    $vals = [];
    foreach ($transaction->worth->currencies() as $curr_id) {
      // @todo running balance means sorting by the same field the view is sorted by.
      $vals[] = [
        'curr_id' => $curr_id,
        'value' => $this->txStorage->runningBalance($fAlias, $curr_id, 'created')
      ];
    }
    return $vals;
  }

  /**
   * helpers
   */
  private function getFirstWalletFieldAlias() {
    $q = $this->view->getQuery();
    foreach ($q->tables as $table_name => $relations) {
      foreach (array_keys($relations) as $alias) {
        $table_info = $q->getTableInfo($alias);
        if ($table_info['table'] == 'mcapi_wallet' && $table_info['join']->leftField == 'wallet_id') {
          foreach ($q->fields as $field_alias => $field_info) {
            if ($field_info['field'] == 'wallet_id') {
              return $field_alias;
            }
          }
        }
        elseif(isset($q->fields['wallet_id'])) {
          return 'wallet_id';
        }
      }
    }
    throw new \Exception('Running balance requires that there be a relationship with the wallet_id field');
  }

}

==> mutual_credit-6.0.0-beta3.tar.gz: mutual_credit/src/Plugin/views/field/RunningBalance.php <==
<?php

namespace Drupal\mcapi\Plugin\views\field;

use Drupal\Mcapi\Functions;
use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\Core\Entity\EntityTypeManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Field handler to provide running balance for a given transaction.
 *
 * @note reads from the transaction index table
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("transaction_running_balance")
 */
class RunningBalance extends FieldPluginBase {

  private $txStorage;
  private $fAlias;

  /**
   * Constructs a Handler object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param EntityTypeManager $entity_type_manager
   *   The EntityTypeManager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->txStorage = $entity_type_manager->getStorage('mc_transaction');
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition,
      $container->get('entity_type.manager')
    );
  }

  function query() {
    $this->addAdditionalFields();
  }


  /**
   * {@inheritDoc}
   */
  public function render(ResultRow $values) {
    $transaction = $this->getEntity($values);
    if (!$transaction) {
      //something wrong here
      throw new \Exception('No entity in views result');
    }
    $wid = $this->{aliases['wallet_id']};
    $created = $values->{$aliases['created']};
    $val = $this->txStorage->runningBalance($wid, 'created', $created);
    return Functions::formatCurrency($val);
  }

}
